@@ -1,13 +1,13 @@
-# Modified 12 Sept 2015 SDH
+# Modified 12 May 2016 SDH
 
 checkArgClassValue <- function(object, expected.class = NULL, expected.values = NULL, expected.range = NULL, warn.only = FALSE) {
 
   # call. set to FALSE so user does not see reference to unknown checkArgClassValue() function.
   if(!is.null(expected.class) && !class(object) %in% expected.class)  {
     if(!warn.only) {
-      stop('Expect class \"', expected.class, '\" for argument ', deparse(substitute(object)), ' but got \"', paste(class(object), collapse = ', '), '\".', call. = FALSE)
+      stop('Expect class \"', paste(expected.class, collapse = ', '), '\" for argument ', deparse(substitute(object)), ' but got \"', paste(class(object), collapse = ', '), '\".', call. = FALSE)
     } else {
-      warning('Expect class \"', expected.class, '\" for argument ', deparse(substitute(object)), ' but got \"', paste(class(object), collapse = ', '), '\".', call. = FALSE)
+      warning('Expect class \"', paste(expected.class, collapse = ', '), '\" for argument ', deparse(substitute(object)), ' but got \"', paste(class(object), collapse = ', '), '\".', call. = FALSE)
     }
   } 
 
